/**
 * MOTECH PLATFORM OPENSOURCE LICENSE AGREEMENT
 *
 * Copyright (c) 2010-11 The Trustees of Columbia University in the City of
 * New York and Grameen Foundation USA.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Grameen Foundation USA, Columbia University, or
 * their respective contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY GRAMEEN FOUNDATION USA, COLUMBIA UNIVERSITY
 * AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GRAMEEN FOUNDATION
 * USA, COLUMBIA UNIVERSITY OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.motechproject.mobile.core.model;

import java.util.Date;

/**
 * Holds IncomingMessage information for data storage and manipulation.
 * Corresponds to a message from handset either sms or openxdata xform
 *
 * Date: Dec 14, 2009
 * @author Joseph Djomeda (joseph@dreamoval.com)
 */
public interface IncomingMessage {


    /*
     *
     * @param id id to set
     */
    public void setId(Long id);

    /*
     * 
     * @return the id
     */
    public Long getId();

    /*
     * @return the content
     */
    public String getContent();

    /*
     * @return the dateCreated
     */
    public Date getDateCreated();

    /*
     * @return the incomingMessageForm
     */
    public IncomingMessageForm getIncomingMessageForm();

    /*
     * @return the incomingMessageResponse
     */
    public IncomingMessageResponse getIncomingMessageResponse();

    /*
     * @return the incomingMsgSession
     */
    public IncomingMessageSession getIncomingMsgSession();

    /*
     * @return the lastModified
     */
    public Date getLastModified();

    /*
     * @return the messageStatus
     */
    public IncMessageStatus getMessageStatus();

    /*
     * @param content the content to set
     */
    public void setContent(String content);

    /*
     * @param dateCreated the dateCreated to set
     */
    public void setDateCreated(Date dateCreated);

    /*
     * @param incomingMessageForm the incomingMessageForm to set
     */
    public void setIncomingMessageForm(IncomingMessageForm incomingMessageForm);

    /*
     * @param incomingMessageResponse the incomingMessageResponse to set
     */
    public void setIncomingMessageResponse(IncomingMessageResponse incomingMessageResponse);

    /*
     * @param incomingMsgSession the incomingMsgSession to set
     */
    public void setIncomingMsgSession(IncomingMessageSession incomingMsgSession);

    /*
     * @param lastModified the lastModified to set
     */
    public void setLastModified(Date lastModified);

    /*
     * @param messageStatus the messageStatus to set
     */
    public void setMessageStatus(IncMessageStatus messageStatus);

    /*
     * Helper method to display string value of all properties of the object
     * @return formated string value of all properties
     */
    @Override
    public String toString();

    /*
     * @return the version
     */
    int getVersion();

    /*
     * @param version the version to set
     */
    void setVersion(int version);
}
